import collections

import agent.config
from common.utils import subprocess2


class Error(Exception):
    pass


class NotAScriptError(Error):
    """Raised when user-data has no shebang"""


Script = collections.namedtuple("Script", "script_type, name, content")


def parse_script(user_data):
    assert user_data is not None
    sw = user_data.startswith
    if not sw("#!"):
        msg = (
            "First line of the user-data script should have the form of a shebang "
            "interpreter directive is as follows:\n"
            "#!interpreter [options]"
        )
        raise NotAScriptError(msg)
    if sw("#!powershell"):
        return Script("powershell", "bootstrap", user_data)
    if sw("#!cmd"):
        return Script("cmd", "bootstrap", user_data)
    return Script("shell", "bootstrap", user_data)


def run_script(path, script_type):
    """Facade function to run scripts"""
    fn = globals().get("run_{}_script".format(script_type))
    if not callable(fn):
        msg = "Unknown script type: {}".format(script_type)
        raise Error(msg)
    return fn(path)


def run_shell_script(path):
    subprocess2.check_output(
        [path], shell=True, timeout=agent.config.workflow_engine["global_timeout"]
    )


def run_cmd_script(path):
    with open(path, "r") as f:
        data = f.read()
    if data.startswith("#!cmd"):
        data = "\n".join(data.splitlines()[1:])
    subprocess2.check_output(
        ["call"], shell=True, timeout=agent.config.workflow_engine["global_timeout"], input=data
    )


def run_powershell_script(path):
    subprocess2.check_output(
        [
            "powershell.exe",
            "-NoProfile",
            "-NonInteractive",
            "-ExecutionPolicy",
            "Bypass",
            "-File",
            path,
        ],
        shell=True,
        timeout=agent.config.workflow_engine["global_timeout"],
    )
