from __future__ import absolute_import

import copy
import json
import os

import celery

import agent.config
from agent.utils import celeryutil
from common.utils import logutil
from common.utils import mixutil
from common.utils import six

LOG = logutil.get_logger(__name__)


@celeryutil.agent_task(name="agent.patch_config")
def patch_config(config, **kwargs):
    config_path = agent.config.workflow_engine["agent_json_path"]
    patch = copy.deepcopy(config)
    origin = {"workflow_engine": {}}
    if os.path.exists(config_path):
        with open(config_path) as fp:
            origin = json.load(fp)

    if patch.get("workflow_engine", {}).get("celery", {}).get("broker_url"):
        LOG.warning("Updating Scalr connection settings")
        origin["workflow_engine"].setdefault("celery", {})
        origin["workflow_engine"]["celery"].setdefault(
            "broker_url", celery.current_app.conf.BROKER_URL
        )
        patch["workflow_engine"]["celery"]["broker_url"] = patch_url(
            origin["workflow_engine"]["celery"]["broker_url"],
            patch["workflow_engine"]["celery"]["broker_url"],
        )

    mixutil.merge_recursive(origin, patch)
    LOG.debug("Writing new configuration: %s", origin)
    with open(config_path, "w+") as fp:
        json.dump(origin, fp)
    LOG.info("New configuration written: %s", config_path)


def patch_url(origin, patch):
    """Updates components in the `origin` URL with the `patch`

    Example:: Update password
    >> _patch_url('amqp://guest:guest@127.0.0.5/%20', 'amqp://:newpassword@/')
    >> 'amqp://guest:newpassword@127.0.0.5/%20'
    """
    origin = six.moves.urllib.parse.urlparse(origin)
    patch = six.moves.urllib.parse.urlparse(patch)
    new = list(origin)
    new[0] = patch.scheme or origin.scheme
    new[1] = "{}:{}@{}:{}".format(
        patch.username or origin.username,
        patch.password or origin.password,
        patch.hostname or origin.hostname,
        patch.port or origin.port,
    )
    # pylint: disable=too-many-function-args
    return six.moves.urllib.parse.urlunparse(new)
