"""
Created on Aug 29, 2012

@author: marat
"""
from __future__ import absolute_import

import agent.tasks.filesystem.tasks as filesystem
from common.utils import logutil
from common.utils import subprocess2
from common.utils.facts import fact

E2LabelCmd = subprocess2.new_command("E2LabelCmd", binary="/sbin/e2label", MAX_LABEL_LENGTH=16)
Resize2FsCmd = subprocess2.new_command("Resize2FsCmd", binary="/sbin/resize2fs")
E2fsckCmd = subprocess2.new_command("E2fsckCmd", binary="/sbin/e2fsck")


LOG = logutil.get_logger(__name__)


class Ext3FileSystem(filesystem.LinuxFileSystem):
    """
    EXT file system management.
    """

    freezable = True
    extendable = True
    umount_on_resize = True

    packages = ["e2fsprogs"]

    def __init__(self, name, *args, **kwds):
        if fact["os"]["family"] == "redhat":
            self.packages.append("e4fsprogs")
        super(Ext3FileSystem, self).__init__(name, *args, **kwds)

    def mkfs(self, device, *args, **kwds):
        """
        Creates new file system on the device.
        """
        args = sorted(list(set(args) | set(("-F", "-q"))))
        return super(Ext3FileSystem, self).mkfs(device, *args, **kwds)

    def extend(self, device, *args, **kwds):
        """
        Extend a logical volume.
        """
        try:
            E2fsckCmd().args("-f", device).check_output()
        except subprocess2.CalledProcessError as e:
            error_msg = str(e)
            if "is mounted" in error_msg or "is in use" in error_msg:
                LOG.debug("e2fsck check for %s is unnecessary " "because it is mounted.", device)
            else:
                raise filesystem.ExtendfsError(device, e)

        try:
            Resize2FsCmd().args(device).check_output()
        except subprocess2.CalledProcessError as e:
            raise filesystem.ExtendfsError(device, e)

    def set_label(self, device, label):
        """
        Applies new label to the device.
        """
        try:
            cmd = E2LabelCmd()
            cmd.args(device)
            cmd.args(label[: E2LabelCmd.MAX_LABEL_LENGTH])
            cmd.check_output()
        except subprocess2.CalledProcessError as e:
            raise filesystem.LabelSetError(device, e)

    def get_label(self, device):
        """
        Returns current label of the device.
        """
        try:
            return E2LabelCmd().args(device).check_output().strip()
        except subprocess2.CalledProcessError as e:
            raise filesystem.LabelGetError(device, e)


class Ext4FileSystem(Ext3FileSystem):
    umount_on_resize = False
