"""
Created on Aug 29, 2012

@author: marat
"""
from __future__ import absolute_import

import re

import agent.tasks.filesystem.tasks as filesystem
from common.utils import subprocess2
from common.utils.facts import fact

if fact["os"]["family"] != "windows":
    from common.utils import linuxutil
else:
    linuxutil = None  # pragma: no cover


XfsAdminCmd = subprocess2.new_command("XfsAdminCmd", binary="/usr/sbin/xfs_admin")
XfsGrowfsCmd = subprocess2.new_command("XfsGrowfsCmd", binary="/usr/sbin/xfs_growfs")
XfsFreezeCmd = subprocess2.new_command("XfsFreezeCmd", binary="/usr/sbin/xfs_freeze")


def check_mpoint(device):
    """
    Returns mount point for the given device.

    :raises: :class:`filesystem.MountRequiredError`
    """
    try:
        return linuxutil.mounts().get_device(device).mpoint
    except KeyError:
        raise filesystem.MountRequiredError(device)


class XfsFileSystem(filesystem.LinuxFileSystem):
    """
    XFS filesystem management.
    """

    freezable = True
    extendable = True
    umount_on_resize = False

    packages = ["xfsprogs"]

    def set_label(self, device, label):
        try:
            XfsAdminCmd().args("-L", label, device).check_output()
        except subprocess2.CalledProcessError as e:
            raise filesystem.LabelSetError(device, e)

    def get_label(self, device):
        try:
            output = XfsAdminCmd().args("-l", device).check_output()
            label_re = re.compile(r'label\s+=\s+"(?P<label>.*)"', re.IGNORECASE)
            match = label_re.search(output)
            return match.group("label") if match else ""
        except subprocess2.CalledProcessError as e:
            raise filesystem.LabelGetError(device, e)

    def extend(self, device, *args, **kwds):
        mpoint = check_mpoint(device)
        try:
            XfsGrowfsCmd().args(mpoint).check_output()
        except subprocess2.CalledProcessError as e:
            raise filesystem.ExtendfsError(device, e)

    def freeze(self, device):
        mpoint = check_mpoint(device)
        try:
            XfsFreezeCmd().args("-f", mpoint).check_output()
        except subprocess2.CalledProcessError as e:
            raise filesystem.FreezeError(device, e)

    def unfreeze(self, device):
        mpoint = check_mpoint(device)
        try:
            XfsFreezeCmd().args("-u", mpoint).check_output()
        except subprocess2.CalledProcessError as e:
            raise filesystem.UnfreezeError(device, e)
