from __future__ import absolute_import

from agent.tasks.service import linux
from common.utils import linuxutil
from common.utils import subprocess2

ChkConfigCmd = subprocess2.new_command("ChkConfigCmd", binary="/sbin/chkconfig")


class RedHatServiceProvider(linux.LinuxServiceProvider):
    def _chkconfig(self, service_name, action):
        cmd_params = (action, service_name)
        if action in ("on", "off", "reset", "resetpriorities"):
            cmd_params = reversed(cmd_params)
        cmd = ChkConfigCmd().args(*cmd_params)
        return cmd.check_output()

    def set_service_enabled(self, name, enabled):
        action = "on" if enabled else "off"
        self._chkconfig(name, action)

    def get_service_enabled(self, name):
        try:
            out = self._chkconfig(name, "--list")
        except subprocess2.CalledProcessError:
            return False
        runlevel_states = dict(x.split(":") for x in out.split()[1:])
        run_level = str(linuxutil.runlevel())
        return runlevel_states[run_level] == "on"
