import agent.tasks.service.tasks as service
from agent.tasks.service import debian as debian_pvd
from common.utils import subprocess2


class UpstartServiceProvider(debian_pvd.DebianServiceProvider):
    """
    Upstart service provider.
    Used for all Ubuntu versions older than 16.04.
    https://wiki.ubuntu.com/SystemdForUpstartUsers#A.2Fetc.2Fdefault_files_which_enable.2Fdisable_jobs
    """

    def _run_service_action(self, name, action):
        self._check_action(action)
        cmd = "service {} {}".format(name, action)
        return subprocess2.run(cmd, shell=True)

    def get_service_status(self, name):
        if "Unknown job" in subprocess2.run("initctl status %s" % name, shell=True).stderr:
            return super(UpstartServiceProvider, self).get_service_status(name)
        out = self._run_service_action(name, "status").stdout
        if "start/running" in out:
            return service.ServiceStatus.running
        elif "stop/waiting" in out:
            return service.ServiceStatus.stopped
        else:
            return service.ServiceStatus.unknown
