import typing as tp

import botocore
import requests


class ProxyConfiguration(botocore.httpsession.ProxyConfiguration):
    """
    This class extends :class:`botocore.httpsession.ProxyConfiguration`
    to provide "no_proxy" support.
    """

    def proxy_url_for(self, url: str) -> tp.Optional[str]:
        """Retrirves the corresponding proxy url for a given url."""
        no_proxy: tp.Optional[str] = self._proxies.get("no_proxy") if self._proxies else None
        if no_proxy:
            no_proxy = no_proxy.strip()

        if no_proxy == "*":
            return None

        safe_url = url.strip().rstrip("/")
        if no_proxy and requests.utils.should_bypass_proxies(safe_url, no_proxy):  # type: ignore
            return None

        result: tp.Optional[str] = super(ProxyConfiguration, self).proxy_url_for(url)
        return result


def patch_boto(no_proxy: bool = True) -> None:
    """Patch boto to add new features.

    Features:
      - no_proxy: Enabled no_proxy support.

      Example:
        >>> config = botocore.config.Config(proxies={"http": "...", "https": "...", "no_proxy": "..."})
        >>> client = boto3.client(config=config)
    """
    if no_proxy:
        botocore.httpsession.ProxyConfiguration = ProxyConfiguration
