__author__ = "Dmitriy Korsakov"

import json
import sys

from common.utils import logutil
from common.utils import subprocess2

LOG = logutil.get_logger(__name__)

RC_NOT_FOUND = 127
SUPPORTED_DOCKER_VERSION = (1, 9, 0)


def get_container_stat(container):
    try:
        info = subprocess2.check_output_host("docker inspect {}".format(container), shell=True)
        info = json.loads(info)[0]
        result = {container: {}}
        result[container]["container_id"] = info["Id"]

        entrypoint = info["Config"]["Entrypoint"] or []
        if not isinstance(entrypoint, list):
            entrypoint = [entrypoint]
        cmd = info["Config"]["Cmd"] or []
        if not isinstance(cmd, list):
            cmd = [cmd]
        result[container]["command"] = " ".join(entrypoint + cmd)

        # from '2016-07-02T01:27:58.148296245Z' to '2016-07-02 01:27:58'
        created = info["Created"]
        created = created[: created.index(".")]  # getting rid of milliseconds
        created = created.replace("T", " ")
        result[container]["created"] = created

        result[container]["image"] = info["Config"]["Image"]
        result[container]["labels"] = info["Config"]["Labels"]
        result[container]["name"] = info["Name"]
        result[container]["network"] = info["HostConfig"]["NetworkMode"]
        result[container]["privileged"] = info["HostConfig"]["Privileged"]

        result[container]["volumes"] = []
        for m in info["Mounts"]:
            result[container]["volumes"].append(
                {"source": m["Source"], "destination": m["Destination"]}
            )

        result[container]["ports"] = {}
        for destination, sources in info["NetworkSettings"]["Ports"].items():
            if sources is None:
                result[container]["ports"][destination] = None
            else:
                result[container]["ports"][destination] = []
                for s in sources:
                    result[container]["ports"][destination].append(
                        {"port": int(s["HostPort"]), "host_ip": s["HostIp"]}
                    )
        return result
    except Exception:
        LOG.debug('Inspecting container "%s" failed: %s', container, sys.exc_info()[1])
        return {}
