from scalarizr.adm.command import Command
from scalarizr.adm.command import CommandError
from agent.scalarizr import msg


class Msgsnd(Command):
    """
    Sends message with given name using given endpoint throughout given queue.
    Parameters are passed in <key>=<value> form or can be loaded from msgfile.

    Usage:
      msgsnd [--queue=<queue>] [--name=<name>] [--msgfile=<msgfile>] [--endpoint=<endpoint>] [<kv>...]

    Options:
      -n <name>, --name=<name>
      -f <msgfile>, --msgfile=<msgfile>
      -e <endpoint>, --endpoint=<endpoint>
      -o <queue>, --queue=<queue>
    """

    def __call__(self, name=None, msgfile=None, endpoint=None, queue=None, kv=None):
        if not msgfile and not name:
            raise Exception('msgfile or name sholuld be presented')
        if kv is None:
            kv = {}
        message = None

        # Message from file
        if msgfile:
            try:
                with open(msgfile, 'r') as f:
                    message = msg.Message.from_json(f.read())
                    if name:
                        message.name = name
            except ValueError:
                raise CommandError('Unknown message format')
            except IOError:
                raise CommandError("Couldn't read message file")

        # Message from command line arguments
        else:
            body = dict(pair.split('=') for pair in kv if '=' in pair)
            message = msg.create(name, body=body)

        msg.send(message, queue=queue, endpoint=endpoint)
        print("Done")


commands = [Msgsnd]
