import json
import os
import sys
import urllib.request

from scalarizr.adm import command
from scalarizr import linux
from scalarizr import util
from common.utils import rpcutil
from agent.scalarizr.hub import hub

POLLING_TIMEOUT = 60
POLLING_INTERVAL = 1


class AgentUpdate(command.Command):
    """
    Updates scalarizr.

    Usage:
      agent-update [--force]

    Options:
      -f, --force  ignore update server restrictions
    """

    def __call__(self, force=False):
        sys.stdout.write('Updating Scalarizr.')
        sys.stdout.flush()
        upd_service = rpcutil.Client(
            'http://localhost:8008/',
            hub.crypto_key)
        try:
            status = upd_service.status()
            if status['candidate'] is None:
                print('\nScalarizr is up-to-date.\n')
                return

            sys.stdout.write('Updating Scalarizr.')
            sys.stdout.flush()
            upd_service.update(force=force, async=True)
        except urllib.request.URLError as e:
            raise command.CommandError(
                '\nError connecting to UpdateClient: "{}".\n'
                'Check that {} service is running.\n'
                .format(
                    e.reason,
                    'ScalrUpdClient' if linux.os.windows_family else 'scalr-upd-client'))

        update_started = [False]
        status = {}

        def _completed():
            sys.stdout.write('.')
            sys.stdout.flush()
            status.update(hub.updateclient.status)
            if status['state'] in ('completed', 'error', 'rollbacked') and update_started[0]:
                return status
            elif not update_started[0] and status['state'].startswith('in-progress'):
                update_started[0] = True
            return False

        try:
            util.wait_until(_completed, sleep=POLLING_INTERVAL, timeout=POLLING_TIMEOUT)
        except Exception as e:
            if 'Timeout:' in str(e):
                status['error'] = str(e)
            else:
                raise

        if status.get('error'):
            raise command.CommandError('\nUpdate failed.\n{}\n'.format(status['error']))
        else:
            installed = upd_service.status()['installed']
            print('\nDone.\nInstalled: {}\n'.format(installed))


commands = [AgentUpdate]
