'''
Created on Mar 1, 2010

@author: marat
'''

import os
import json
from xml.etree import ElementTree
import logging

from agent.scalarizr.hub import hub

from scalarizr import linux
from scalarizr.handlers.block_device import BlockDeviceHandler
from scalarizr.storage2.volumes import ebs


LOG = logging.getLogger(__name__)


def get_handlers():
    return [EbsHandler()]

class EbsHandler(BlockDeviceHandler):
    ec2launch_mapping_config = r'C:\ProgramData\Amazon\EC2-Windows\Launch\Config\DriveLetterMappingConfig.json'
    ec2config_config_xml = r'C:\Program Files\Amazon\Ec2ConfigService\Settings\config.xml'

    def __init__(self):
        BlockDeviceHandler.__init__(self, 'ebs')
        hub.on(init=self.on_ebs_init)

    def get_devname(self, devname):
        return ebs.device2name(devname)

    def on_ebs_init(self):
        if linux.os.windows:
            if os.path.exists(self.ec2launch_mapping_config):
                self._disable_ec2launch_drive_letter_mapping()
            elif os.path.exists(self.ec2config_config_xml):
                self._disable_ec2config_drive_plugins()

    def _disable_ec2config_drive_plugins(self):
        LOG.debug('Disabling ec2config drive plugins')
        with open(self.ec2config_config_xml, 'r') as fp:
            raw = fp.read()
        if not raw:
            return

        config_xml = ElementTree.fromstring(raw)
        for plugin in config_xml.findall('./Plugins/'):
            name = plugin.find('Name')
            if name is not None and name.text in ['Ec2SetDriveLetter', 'Ec2InitializeDrives']:
                state = plugin.find('State')
                state.text = 'Disabled'

        with open(self.ec2config_config_xml, 'w') as fp:
            fp.write(ElementTree.tostring(config_xml).decode('utf-8'))

        linux.system(('net', 'stop', 'ec2config'), raise_exc=False)
        linux.system(('net', 'start', 'ec2config'), raise_exc=False)

    def _disable_ec2launch_drive_letter_mapping(self):
        LOG.debug('Disabling ec2launch drive letter mapping')

        with open(self.ec2launch_mapping_config, 'w') as fp:
            json.dump({"driveLetterMapping": []}, fp)
