'''
Created on Dec 11, 2009

@author: Dmytro Korsakov
@author: marat
'''

from agent.scalarizr.hub import hub

# Core
from scalarizr.handlers import Handler
from scalarizr.handlers import Messages

# Stdlibs
import os
import shutil
import logging


# TODO: Configurator
# TODO: handle IPAddressChanged


def get_handlers():
    return [IpListBuilder()]


class IpListBuilder(Handler):
    name = "ip_list_builder"
    _logger = None
    _base_path = None

    def __init__(self):
        self._logger = logging.getLogger(__name__)
        hub.on(init=self.on_init, reload=self.on_reload)
        self.on_reload()

    def on_init(self, *args, **kwargs):
        hub.on("start", self.on_start)
        hub.on("before_host_up", self.on_before_host_up)

    def on_reload(self):
        self._base_path = hub.ip_list_builder.base_path
        self._base_path = self._base_path.replace('$etc_path', hub.etc_dir)
        self._base_path = os.path.normpath(self._base_path)

    def accept(self, message, queue, behaviour=None, platform=None, os=None, dist=None):
        return message.name == Messages.HOST_UP \
                or message.name == Messages.HOST_DOWN \
                or message.name == Messages.BEFORE_HOST_TERMINATE \
                or message.name == Messages.REBOOT_START \
                or message.name == Messages.REBOOT_FINISH

    def on_start(self, *args):
        if hub.state == 'running':
            self._rebuild()

    def on_before_host_up(self, *args):
        self._rebuild()

    def _rebuild(self):
        """
        Build current hosts structure on farm
        """
        self._logger.debug('Rebuild farm hosts structure')
        if os.path.exists(self._base_path):
            shutil.rmtree(self._base_path)

        for role in hub.queryenv.list_roles():
            for host in role.hosts:
                ipaddr = host.internal_ip or host.external_ip
                if not ipaddr:
                    continue
                self._modify_tree(
                    role.name,
                    role.behaviour,
                    ipaddr,
                    modfn=self._create_file)

    def on_HostUp(self, message):
        behaviour = message.behaviour
        ip = message.local_ip or message.remote_ip
        rolename = message.role_name
        if not ip:
            self._logger.debug('Skip host without ip')
            return

        if ip and rolename and behaviour:
            self._logger.debug("Add host (role_name: %s, behaviour: %s, ip: %s)",
                            rolename, behaviour, ip)
            self._modify_tree(rolename, behaviour, ip,
                            modfn=self._create_file)

    def on_HostDown(self, message):
        behaviour = message.behaviour
        ip = message.local_ip or message.remote_ip
        rolename = message.role_name
        if not ip:
            self._logger.debug('Skip host without ip')
            return

        if ip and rolename and behaviour:
            self._logger.debug("Remove host (role_name: %s, behaviour: %s, ip: %s)",
                                            rolename, behaviour, ip)
            self._modify_tree(rolename, behaviour, ip,
                            modfn=self._remove_file)

    on_BeforeHostTerminate = on_HostDown

    on_RebootStart = on_HostDown

    on_RebootFinish = on_HostUp

    def _modify_tree(self, rolename, behaviours, ip, modfn=None):
        # Touch/Unlink %role_name%/xx.xx.xx.xx
        modfn(os.path.join(self._base_path, rolename, ip))

        for behaviour in behaviours:
            # Touch/Unlink %behaviour%/xx.xx.xx.xx
            modfn(os.path.join(self._base_path, behaviour, ip))

    def _create_dir(self, d):
        if not os.path.exists(d):
            try:
                self._logger.debug("Create dir %s", d)
                os.makedirs(d, 0o755)
            except OSError as e:
                self._logger.exception(e)

    def _create_file(self, f):
        self._create_dir(os.path.dirname(f))
        try:
            self._logger.debug("Touch file %s", f)
            open(f, 'w').close()
            os.chmod(f, 0o644)
        except OSError as e:
            self._logger.error(e)

    def _remove_dir(self, d):
        if os.path.exists(d) and not os.listdir(d):
            try:
                self._logger.debug("Remove dir %s", d)
                os.rmdir(d)
            except OSError as e:
                self._logger.error(e)

    def _remove_file(self, f):
        if os.path.exists(f):
            try:
                self._logger.debug("Remove file %s", f)
                os.remove(f)
            except OSError as e:
                self._logger.error(e)
        self._remove_dir(os.path.dirname(f))

