import os

from agent.scalarizr.hub import hub
from common import types

from scalarizr import platform


def get_platform():
    return AzurePlatform()


class AzurePlatform(platform.Platform):

    name = 'azure'
    features = [platform.PlatformFeatures.VOLUMES]

    def get_cloud_server(self):
        """
        Returns cloud server.

        :rtype: :class:`common.types.data.AzureCloudServer`
        """
        cloud_server_id = hub.general.cloud_server_id
        location = hub.general.region

        resource_group = self.get_access_data('resource_group')
        subscription_id = self.get_access_data('subscription_id')

        vm_id = (
            '/subscriptions/{subscription_id}/resourceGroups/{resource_group}'
            '/providers/Microsoft.Compute/virtualMachines/{cloud_server_id}'
        ).format(
            subscription_id=subscription_id,
            resource_group=os.path.basename(resource_group),
            cloud_server_id=cloud_server_id
        )
        return types.AzureCloudServer(dict(
            id=vm_id,
            location=location,
            resource_group=resource_group
        ))

    def get_cloud_credentials(self):
        """
        Returns credentials from Azure Cloud.

        :rtype: :class:`common.types.data.CloudCredentials`
        """
        access_data = self.get_access_data()
        return types.AzureCloudCredentials(dict(
            tenant_id=access_data['tenant_name'],
            subscription_id=access_data['subscription_id'],
            client_id=access_data['client_id'],
            secret_key=access_data['secret_key'],
            account_type=access_data.get('account_type'),
            proxy=access_data.get('proxy')
        ))
