from scalarizr.platform import Ec2LikePlatform, PlatformFeatures

from common import types


#Platform configuration options
OPT_ACCOUNT_ID = "account_id"
OPT_KEY = "key"
OPT_KEY_ID = "key_id"
OPT_EC2_CERT_PATH = "ec2_cert_path"
OPT_CERT_PATH = "cert_path"
OPT_PK_PATH = "pk_path"



#User data options
UD_OPT_S3_BUCKET_NAME = "s3bucket"



def get_platform():
    return Ec2Platform()


class Ec2Platform(Ec2LikePlatform):
    name = "ec2"

    _userdata_key = "latest/user-data"


    instance_store_devices = (
            '/dev/sda2', '/dev/sdb', '/dev/xvdb',
            '/dev/sdc', '/dev/xvdc',
            '/dev/sdd', '/dev/xvdd',
            '/dev/sde', '/dev/xvde'
    )

    _logger = None
    _cnf = None

    features = [PlatformFeatures.SNAPSHOTS, PlatformFeatures.VOLUMES]

    def get_account_id(self):
        return self.get_access_data("account_id")

    def get_access_keys(self):
        return (self.get_access_data("key_id"), self.get_access_data("key"))

    @property
    def cloud_storage_path(self):
        ret = Ec2LikePlatform.cloud_storage_path.fget(self)
        if not ret:
            bucket = self.get_user_data(UD_OPT_S3_BUCKET_NAME) or ''
            ret = 's3://' + bucket
        return ret

    def get_cloud_server(self):
        """
        Returns cloud server.

        :rtype: :class:`common.types.EC2CloudServer`
        """
        return types.EC2CloudServer({
            'id': self.get_instance_id(),
            'location': self.get_region(),
            'availability_zone': self.get_avail_zone()
        })

    def get_cloud_credentials(self):
        """
        Returns credentials from Azure Cloud.

        :rtype: :class:`common.types.data.CloudCredentials`
        """
        access_key, secret_key = self.get_access_keys()
        account_id = self.get_account_id()
        proxy = self.get_access_data().get('proxy')
        return types.AwsCloudCredentials({
            'account_id': account_id,
            'access_key': access_key,
            'secret_key': secret_key,
            'proxy': proxy,
        })
