import os
import logging

from scalarizr import storage2
from scalarizr.storage2.volumes import azure

from common import types


LOG = logging.getLogger(__name__)


class AzureVHDVolume(azure.BaseAzureVolume):
    """
    Azure VHD (unmanaged) volume class.
    """

    def __init__(self, *args, **kwargs):
        super(AzureVHDVolume, self).__init__(*args, **kwargs)

        self._volume = types.AzureVHDVolume({
            'id': self.id,
            'scalr_storage_id': self.scalr_storage_id,
            'vhd_uri': self.vhd_uri,
            'resource_group': os.path.basename(self.resource_group),
            'tags': self.tags,
            'type': self.type,
            'snap': self.snap or None,
            'device': self.device,
            'fstype': self.fstype,
            'mpoint': self.mpoint,
            'mount_options': self.mount_options,
            'recreate_if_missing': self.recreate_if_missing,
            'template': self.template,
            'size': int(self.size),
            'label': getattr(self, 'label', None),  # windows-only
        })


storage2.volume_types['azure_vhd'] = AzureVHDVolume
